/**
* Navision Object Checkout v0.8 installation script.
* @version 0.8
* @author B3 Technologies for Business
*/

--Update the next line with the actual Navision DB name.
USE [<DB NAME>];  

IF EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[NOC_UPDATE]'))
DROP TRIGGER [dbo].[NOC_UPDATE];
GO

CREATE TRIGGER [dbo].[NOC_UPDATE]
	ON [dbo].[Object] INSTEAD OF UPDATE AS
BEGIN	
	DECLARE @checkoutUserID AS VARCHAR(50), @errorMsg AS VARCHAR(8000);	
	DECLARE @updObjType AS INT, @updObjNo AS INT, @updObjName AS VARCHAR(50), @updObjCaption AS VARCHAR(80),
		@updUserID AS VARCHAR(50), @shortUpdUserID AS VARCHAR(20), @i0 AS INT;
	DECLARE @orgVersList AS VARCHAR(250), @orgDate AS DATETIME, @orgTime AS DATETIME, @orgBLOBSize AS INT, @newObject AS INT;	
	--<Handle renumbering>
	DECLARE @prevObjType AS INT, @prevObjNo AS INT, @prevCompName AS VARCHAR(50);
	SET @prevObjType = (SELECT [Type] FROM deleted);
	SET @prevObjNo = (SELECT [ID] FROM deleted);
	SET @prevCompName = (SELECT [Company Name] FROM deleted);
	--SET @prevObjName = (SELECT [Name] FROM deleted);
	--</Handle renumbering>

	SET @updObjType = (SELECT [Type] FROM inserted);
	SET @updObjNo = (SELECT [ID] FROM inserted);
	SET @updObjName = (SELECT [Name] FROM inserted);
	--SET @updObjCaption = (SELECT [Caption] FROM inserted);
	SET @updObjCaption = '';
	SET @updUserID = UPPER(SUSER_SNAME());
	SET @i0 = CHARINDEX('\', @updUserID);
	IF (@i0 <> 0) BEGIN
		SET @shortUpdUserID = SUBSTRING(@updUserID, @i0 + 1, 20);
	END ELSE BEGIN
		SET @shortUpdUserID = @updUserID;
	END;

	SET @checkoutUserID = 
		(SELECT [User ID] FROM [dbo].[Navision Object Checkout Entry]
			WHERE (([Object Type] = @prevObjType) AND ([Object No_] = @prevObjNo))
		);	

	IF (@updObjType >= 1) AND (@updObjType <= 7) BEGIN  --2008-10-10: VJ - check if object type is within supported range 		
		IF (@checkoutUserID IS NOT NULL) BEGIN	
			IF (@checkoutUserID <> @shortUpdUserID) BEGIN
				SET @errorMsg = N'Object "' + @updObjName + N'" has been checked out by ' + @checkoutUserID + '.';
				RAISERROR(@errorMsg, 11, 1);
			END 
		END ELSE BEGIN
		
			SELECT @newObject = 0, @orgVersList = [Version List], @orgDate = [Date], @orgTime = [Time], @orgBLOBSize = [BLOB Size]
				FROM deleted;
			IF (@orgVersList IS NULL) BEGIN
				SET @newObject = 1;
				SET @orgVersList = '';
				SET @orgDate = '1753-01-01 00:00:00';
				SET	@orgTime = '1753-01-01 00:00:00';
				SET @orgBLOBSize = 0;
			END;
			INSERT [dbo].[Navision Object Checkout Entry] 
				([Object Type], [Object No_], [Object Name], [Object Caption], [User ID], [Checkout Date_Time],
				[Checked-out Version], [Checked-out Date], [Checked-out Time], [Checked-out BLOB Size], [New Object], [Check-out Type])
				VALUES(@updObjType, @updObjNo, @updObjName, @updObjCaption, @shortUpdUserID, GETDATE(),
					@orgVersList, @orgDate, @orgTime, @orgBLOBSize, @newObject, 1);			
			
			-->>2008-06-13
			DECLARE @lastHistEntryNo AS INT;
			SET @lastHistEntryNo = COALESCE((SELECT MAX([Entry No_]) FROM [dbo].[NOC History Entry]), 0) 
				+ 1;
			INSERT [dbo].[NOC History Entry] 
				([Entry No_], [Action Type],
				[Object Type], [Object No_], [Object Name], [Object Caption], [User ID], [Action Date_Time],
				[Object Version], [Object Date], [Object Time], [Object BLOB Size], [New Object], [Check-out Type], 
				[Renumbered To No_],
				[Object Data])			
				VALUES(@lastHistEntryNo, 1, --action type 1: "check-out"
					@updObjType, @updObjNo, @updObjName, @updObjCaption, @shortUpdUserID, GETDATE(),
					@orgVersList, @orgDate, @orgTime, @orgBLOBSize, @newObject, 1, --check-out type 1: auto
					0,  -- NOC 0.8
					NULL
					);
			UPDATE [dbo].[NOC History Entry] SET [Object Data] = (SELECT [BLOB Reference] FROM deleted)
				WHERE ([Entry No_] = @lastHistEntryNo);
		END;
	END;  --end IF (@updObjType >= 1) AND (@updObjType <= 7) BEGIN  --2008-10-10: VJ - check if object type is within supported range 

	--Actual update - since this is an "INSTEAD OF" trigger, we have to do the UPDATE ourselves
	
	UPDATE [dbo].[Object] SET 
			[Object].[Type] = inserted.[Type],
			[Object].[Company Name] = inserted.[Company Name],
			[Object].[ID] = inserted.[ID],
			[Object].[Name] = inserted.[Name],
			[Object].[Modified] = inserted.[Modified],
			[Object].[Compiled] = inserted.[Compiled],			
			[Object].[BLOB Reference] = inserted.[BLOB Reference], -- (!)
			[Object].[BLOB Size] = inserted.[BLOB Size], -- (!)
			[Object].[DBM Table No_] = inserted.[DBM Table No_],
			[Object].[Date] = inserted.[Date],
			[Object].[Time] = inserted.[Time],
			[Object].[Version List] = inserted.[Version List]
	FROM [dbo].[Object] INNER JOIN inserted ON
		(([Object].[Type] = @prevObjType) --deleted.[Type] 
		AND ([Object].[ID] = @prevObjNo) --deleted.[ID])
		AND ([Object].[Company Name] = @prevCompName))  --deleted.[Company Name]))
	
	IF (@updObjType >= 1) AND (@updObjType <= 7) BEGIN
		IF (@updObjNo <> @prevObjNo) BEGIN  --Object renumbered
			--SET @errorMsg = N'Attempt to renumber object: ' + @updObjName + N'" has been checked out by ' + @checkoutUserID + '.';		
			--RAISERROR(@errorMsg, 11, 1);
			UPDATE [dbo].[Navision Object Checkout Entry] 
				SET [Object No_] = @updObjNo
				WHERE (([Object Type] = @prevObjType) AND ([Object No_] = @prevObjNo));
			
			SET @lastHistEntryNo = COALESCE((SELECT MAX([Entry No_]) FROM [dbo].[NOC History Entry]), 0) + 1;
			SELECT @newObject = 0, @orgVersList = [Version List], @orgDate = [Date], @orgTime = [Time], @orgBLOBSize = [BLOB Size]
				FROM deleted;		
			INSERT [dbo].[NOC History Entry] 
				([Entry No_], [Action Type],
				[Object Type], [Object No_], [Object Name], [Object Caption], [User ID], [Action Date_Time],
				[Object Version], [Object Date], [Object Time], [Object BLOB Size], [New Object], [Check-out Type], 
				[Renumbered To No_],
				[Object Data])			
			VALUES(@lastHistEntryNo, 3, --action type 3: "Renumber"
					@prevObjType, @prevObjNo, @updObjName, @updObjCaption, @shortUpdUserID, GETDATE(),
					@orgVersList, @orgDate, @orgTime, @orgBLOBSize, @newObject, 1, --check-out type 1: auto
					@updObjNo,  -- NOC 0.8
					NULL
					);
		END;  --end (2) IF (@updObjType >= 1) AND (@updObjType <= 7) BEGIN 
	END
END;
GO
-- ---------------


IF EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[NOC_INSERT]'))
DROP TRIGGER [dbo].[NOC_INSERT];
GO

CREATE TRIGGER [dbo].[NOC_INSERT]
	ON [dbo].[Object] FOR INSERT AS
BEGIN	
	DECLARE @checkoutUserID AS VARCHAR(50), @errorMsg AS VARCHAR(8000);	
	DECLARE @updObjType AS INT, @updObjNo AS INT, @updObjName AS VARCHAR(50), @updObjCaption AS VARCHAR(80),
		@updUserID AS VARCHAR(50), @shortUpdUserID AS VARCHAR(20), @i0 as INT;
	DECLARE @orgVersList AS VARCHAR(250), @orgDate AS DATETIME, @orgTime AS DATETIME, @orgBLOBSize AS INT, @newObject AS INT;
	
	SET @updObjType = (SELECT [Type] FROM inserted);
	SET @updObjNo = (SELECT [ID] FROM inserted);
	SET @updObjName = (SELECT [Name] FROM inserted);
	--SET @updObjCaption = (SELECT [Caption] FROM inserted);
	SET @updObjCaption = '';
	SET @updUserID = UPPER(SUSER_SNAME());
	SET @i0 = CHARINDEX('\', @updUserID);
	IF (@i0 <> 0) BEGIN
		SET @shortUpdUserID = SUBSTRING(@updUserID, @i0 + 1, 20);
	END ELSE BEGIN
		SET @shortUpdUserID = @updUserID;
	END;

	SET @checkoutUserID = 
		(SELECT [User ID] FROM [dbo].[Navision Object Checkout Entry]
			WHERE (([Object Type] = @updObjType) AND ([Object No_] = @updObjNo))
		);	

	IF (@updObjType >= 1) AND (@updObjType <= 7) BEGIN
		IF (@checkoutUserID IS NOT NULL) BEGIN	
			IF (@checkoutUserID <> @shortUpdUserID) BEGIN
				SET @errorMsg = N'Object "' + @updObjName + N'" has been checked out by ' + @checkoutUserID + '.';
				RAISERROR(@errorMsg, 11, 1);
			END
		END ELSE BEGIN
			SET @newObject = 1;
			SELECT @orgVersList = [Version List], @orgDate = [Date], @orgTime = [Time], @orgBLOBSize = [BLOB Size]
				FROM inserted;
			IF (@orgVersList IS NULL) BEGIN			
				SET @orgVersList = '';
				SET @orgDate = '1753-01-01 00:00:00';
				SET	@orgTime = '1753-01-01 00:00:00';
				SET @orgBLOBSize = 0;
			END;
			
			INSERT [dbo].[Navision Object Checkout Entry] 
				([Object Type], [Object No_], [Object Name], [Object Caption], [User ID], [Checkout Date_Time],
				[Checked-out Version], [Checked-out Date], [Checked-out Time], [Checked-out BLOB Size], [New Object], 
				[Check-out Type])
				VALUES(@updObjType, @updObjNo, @updObjName, @updObjCaption, @shortUpdUserID, GETDATE(),
					@orgVersList, @orgDate, @orgTime, @orgBLOBSize, @newObject, 1);
					
			DECLARE @lastHistEntryNo AS INT;
			SET @lastHistEntryNo = COALESCE((SELECT MAX([Entry No_]) FROM [dbo].[NOC History Entry]), 0) 
				+ 1;
			INSERT [dbo].[NOC History Entry] 
				([Entry No_], [Action Type],
				[Object Type], [Object No_], [Object Name], [Object Caption], [User ID], [Action Date_Time],
				[Object Version], [Object Date], [Object Time], [Object BLOB Size], [New Object], [Check-out Type], 
				[Renumbered To No_],  --NOC 0.8
				[Object Data])			
				VALUES(@lastHistEntryNo, 1, --action type 1: "check-out"
					@updObjType, @updObjNo, @updObjName, @updObjCaption, @shortUpdUserID, GETDATE(),
					@orgVersList, @orgDate, @orgTime, @orgBLOBSize, @newObject, 1, --check-out type 1: auto,
					0,  --NOC 0.8
					NULL
					);		
		END;
	END;
END;

GO
